Poglite = RegisterMod("Poglite",1)
local game = Game()
local SaveState = {}

---------------------------------------------------------------
---------------------------Settings----------------------------

-- You can edit these settings directly from this file if you want
-- but I highly recommend you have Mod Config Menu installed to change these instead

local PogliteSettings = {
	["BlindBypass"] = false, --Bypass curse of the blind
	["QualityThreshold"] = 3, --Minimum quality required for pog to activate
	["IgnoreQuality"] = false, --Ignore items qualities altogether
}

-- Below is a list to determine preset pog values
-- 1 for Pog, 2 for Not Pog

local CustomPogList = {
	--["Cricket's Head"] = 1,
	--["Magic Mushroom"] = 1,
	--["Sacred Heart"] = 1,
	--["Brimstone"] = 1,
}

---------------------------------------------------------------
------------------------API Functions--------------------------

Poglite.ModCostumes = {}

-- Add pog costumes for modded characters
		-- key (string/int): unique id, name it anything ex. "MyModPog"
		-- playerType (int): playertype id
		-- pogCostumeId (int): id for the pog costume
	-- Optional: 
		-- removeCostumeId (int): id for a costume to be temporarily removed while pog is active
		-- active (bool): set to false to disable this costume
function Poglite:AddPogCostume(key,playerType,pogCostumeId,removeCostumeId,active)

	if active == nil then active = true end
	removeCostumeId = removeCostumeId or nil
	Poglite.ModCostumes[key] = {playerType,pogCostumeId,removeCostumeId,active}
end

-- Global pog boolean
Poglite.WePoggin = false

---------------------------------------------------------------
------------------------Mod Config Menu------------------------

function Poglite:ModConfigInit()
	if ModConfigMenu then

		--[[local categoryToChange = ModConfigMenu.GetCategoryIDByName(PogliteName)
		if categoryToChange then
		ModConfigMenu.MenuData[categoryToChange] = {}
		ModConfigMenu.MenuData[categoryToChange].Name = tostring(PogliteName)
		ModConfigMenu.MenuData[categoryToChange].Subcategories = {}
		end]]
		
		--Get max collectible id
		function Poglite:GetMaxCollectibleID()
			local id = CollectibleType.NUM_COLLECTIBLES-1
			local step = 16
			while step > 0 do
				if Isaac.GetItemConfig():GetCollectible(id+step) ~= nil then
					id = id + step
				else
					step = step // 2
				end
			end

			return id
		end

		local PogliteName = "POG Mod"

		ModConfigMenu.UpdateCategory(PogliteName, {
			Info = {"POG for Good Items settings.",}
		})
		
		--Title
		
		ModConfigMenu.AddText(PogliteName, "Settings", function() return "POG for Good Items" end)
		ModConfigMenu.AddSpace(PogliteName, "Settings")

		-- Settings
		ModConfigMenu.AddSetting(PogliteName, "Settings", 
			{
				Type = ModConfigMenu.OptionType.BOOLEAN,
				CurrentSetting = function()
					return PogliteSettings["BlindBypass"]
				end,
				Display = function()
					local onOff = "False"
					if PogliteSettings["BlindBypass"] then
						onOff = "True"
					end
					return 'Bypass "Curse of the Blind": ' .. onOff
				end,
				OnChange = function(currentBool)
					PogliteSettings["BlindBypass"] = currentBool
				end,
				Info = {"Pog will activate during Curse of the Blind"}
			})
		
		ModConfigMenu.AddSetting(PogliteName, "Settings", 
			{
				Type = ModConfigMenu.OptionType.NUMBER,
				CurrentSetting = function()
					return PogliteSettings["QualityThreshold"]
				end,
				Minimum = 0,
				Maximum = 4,
				Display = function()
					return "Quality Check: " .. PogliteSettings["QualityThreshold"]
				end,
				OnChange = function(currentNum)
					PogliteSettings["QualityThreshold"] = currentNum
				end,
				Info = {"The minimum item quality needed for pog to activate"}
			})
			
		ModConfigMenu.AddSpace(PogliteName, "Settings")
		
		ModConfigMenu.AddSetting(PogliteName, "Settings", 
			{
				Type = ModConfigMenu.OptionType.BOOLEAN,
				CurrentSetting = function()
					return PogliteSettings["IgnoreQuality"]
				end,
				Display = function()
					local onOff = "False"
					if PogliteSettings["IgnoreQuality"] then
						onOff = "True"
					end
					return 'Ignore Quality: ' .. onOff
				end,
				OnChange = function(currentBool)
					PogliteSettings["IgnoreQuality"] = currentBool
				end,
				Info = {"Pogs will ignore the quality system altogether."}
			})
			
			
		--Customization options
		
		local savePos = 0
		local hunPos = 1 
		
		for id = 1, Poglite:GetMaxCollectibleID() do
		
			local item = Isaac.GetItemConfig():GetCollectible(id)
			--print(id)
			
			savePos = savePos + 1 
			if savePos == 101 then
				hunPos = hunPos + 1
				savePos = 1
			end
			
			local ctgName = tostring(hunPos * 100 - 99) .. '-' .. tostring(hunPos * 100)
			
			if id == 1 then
			ModConfigMenu.AddText(PogliteName, ctgName, function() return "Customize settings for any item" end)
			ModConfigMenu.AddSpace(PogliteName, ctgName)
			end
			
			if item ~= nil then
				
				local thisName = item.Name
				local tableIndex = thisName
				
				--funny exceptions (for when they gave items the exact same name thanks guys)
				if id == 120 then
					thisName = "Odd Mushroom (Thin)"
					tableIndex = thisName
				elseif id == 121 then
					thisName = "Odd Mushroom (Large)"
					tableIndex = thisName
				elseif id == 656 then
					thisName = "Damocles (Passive)"
					tableIndex = "Damocles"
				end
				
				--print(tableIndex)
				if CustomPogList[tableIndex] == nil then
					table.insert(CustomPogList,tableIndex)
					CustomPogList[tableIndex] = 0
				end
				
				ModConfigMenu.AddSetting(PogliteName, ctgName, 
				{
					Type = ModConfigMenu.OptionType.NUMBER,
					CurrentSetting = function()
						return CustomPogList[tableIndex]
					end,
					Minimum = 0,
					Maximum = 2,
					Display = function()
						local nChoice = "default"
						if CustomPogList[tableIndex] == 1 then
							nChoice = "pog"
						elseif CustomPogList[tableIndex] == 2 then
							nChoice = "not pog"
						end
						return id .. '     ' .. thisName .. ': ' .. nChoice
					end,
					OnChange = function(currentNum)
						CustomPogList[tableIndex] = currentNum
					end,
					Info = {thisName .. " (quality " .. item.Quality .. ")"}
				})
			end
		end
	end
end

---------------------------------------------------------------
---------------------------Savedata----------------------------

local json = require("json")

function Poglite:SaveGame()
	--print("save")
	SaveState.Settings = {}
	SaveState.Items = {}
	
	for i, v in pairs(PogliteSettings) do
		SaveState.Settings[tostring(i)] = PogliteSettings[i]
	end
	for i, v in pairs(CustomPogList) do
		SaveState.Items[tostring(i)] = CustomPogList[i]
	end
    Poglite:SaveData(json.encode(SaveState))
end
Poglite:AddCallback(ModCallbacks.MC_PRE_GAME_EXIT, Poglite.SaveGame)

function Poglite:OnGameStart(isSave)
	--print("load")
	
    if Poglite:HasData() then	
		SaveState = json.decode(Poglite:LoadData())	
		
        for i, v in pairs(SaveState.Settings) do
			PogliteSettings[tostring(i)] = SaveState.Settings[i]
		end
		for i, v in pairs(SaveState.Items) do
			CustomPogList[tostring(i)] = SaveState.Items[i]
		end
		
		if CustomPogList["Odd Mushroom"] ~= nil then
			CustomPogList["Odd Mushroom"] = 0
		end
    end
end
Poglite:AddCallback(ModCallbacks.MC_POST_GAME_STARTED, Poglite.OnGameStart)

---------------------------------------------------------------
------------------------------Main-----------------------------

Poglite.COSTUME_DEFAULTPOG = Isaac.GetCostumeIdByPath("gfx/characters/isaacpog.anm2")
Poglite.COSTUME_AZAZELPOG = Isaac.GetCostumeIdByPath("gfx/characters/azazelpog.anm2")
Poglite.COSTUME_AZAZELPOGB = Isaac.GetCostumeIdByPath("gfx/characters/azazelpogb.anm2")
Poglite.COSTUME_KEEPERPOGB = Isaac.GetCostumeIdByPath("gfx/characters/keeperpogb.anm2")
Poglite.COSTUME_BLUEBABYPOGB = Isaac.GetCostumeIdByPath("gfx/characters/bluebabypogb.anm2")
Poglite.COSTUME_PAINPOG = Isaac.GetCostumeIdByPath("gfx/characters/painpog.anm2")
Poglite.COSTUME_LILITHPOGB = Isaac.GetCostumeIdByPath("gfx/characters/lilithpogb.anm2")
Poglite.COSTUME_LAZARUS2POGB = Isaac.GetCostumeIdByPath("gfx/characters/lazarus2pogb.anm2")
Poglite.COSTUME_JACOBPOGB = Isaac.GetCostumeIdByPath("gfx/characters/jacobpogb.anm2")
Poglite.COSTUME_JACOB2POGB = Isaac.GetCostumeIdByPath("gfx/characters/jacob2pogb.anm2")
Poglite.COSTUME_SAMSONPOGB = Isaac.GetCostumeIdByPath("gfx/characters/samsonpogb.anm2")

local weirdExtra

--Add or remove POG costume
function Poglite:DoCostume(apply)
	result = false
	
	for playerNum = 1, game:GetNumPlayers() do
		local player = game:GetPlayer(playerNum)
		local pogCostume = Poglite.COSTUME_DEFAULTPOG
		local removeCostume = nil
		
		--AZAZEL
		if Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_AZAZEL then
			pogCostume = Poglite.COSTUME_AZAZELPOG
		--AZAZEL B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_AZAZEL_B then
			pogCostume = Poglite.COSTUME_AZAZELPOGB
		--KEEPER B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_KEEPER_B then
			pogCostume = Poglite.COSTUME_KEEPERPOGB
			removeCostume = Isaac.GetCostumeIdByPath("gfx/characters/character_b13_keeper.anm2")
		--BLUE BABY B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_XXX_B then
			pogCostume = Poglite.COSTUME_BLUEBABYPOGB
			removeCostume = Isaac.GetCostumeIdByPath("gfx/characters/character_b05_bluebaby.anm2")
		--CAIN B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_CAIN_B then
			pogCostume = Poglite.COSTUME_PAINPOG
		--LILITH B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_LILITH_B then
			pogCostume = Poglite.COSTUME_LILITHPOGB
		--LAZARUS 2 B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_LAZARUS2_B then
			pogCostume = Poglite.COSTUME_LAZARUS2POGB
			removeCostume = Isaac.GetCostumeIdByPath("gfx/characters/character_b09_lazarus2.anm2")
		--JACOB 2 B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_JACOB2_B then
			pogCostume = Poglite.COSTUME_JACOB2POGB
			removeCostume = Isaac.GetCostumeIdByPath("gfx/characters/character_b17_jacob2.anm2")
		--JACOB B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_JACOB_B then
			pogCostume = Poglite.COSTUME_JACOBPOGB
			removeCostume = Isaac.GetCostumeIdByPath("gfx/characters/character_b17_jacob.anm2")
		--SAMSON B
		elseif Isaac.GetPlayer(playerNum):GetPlayerType() == PlayerType.PLAYER_SAMSON_B then
			pogCostume = Poglite.COSTUME_SAMSONPOGB
			removeCostume = Isaac.GetCostumeIdByPath("gfx/characters/character_b07_samson.anm2")
		end
		
		--CUSTOM
		for k, v in pairs(Poglite.ModCostumes) do	
			if Isaac.GetPlayer(playerNum):GetPlayerType() == v[1] then 
	
				--print(k,v[1],v[2],v[3],v[4])
				if v[3] ~= nil then
					removeCostume = v[3]
				end

				if not v[4] then 
					pogCostume = nil
				else
					pogCostume = v[2]
					break
				end
			end
		end
		
		--apply
		if pogCostume ~= nil then
			if apply then
				if removeCostume ~= nil then
					player:TryRemoveNullCostume(removeCostume)
				end
			
				player:AddNullCostume(pogCostume)		
				result = true
			else
				if removeCostume ~= nil then
					player:AddNullCostume(removeCostume)	
				end
			
				player:TryRemoveNullCostume(pogCostume)
				
				if weirdExtra ~= nil then
					player:TryRemoveNullCostume(weirdExtra)
				end
				
				if weirdExtra ~= pogCostume then
					weirdExtra = pogCostume
				end
			
				result = false
			end
		end
	end
	
	Poglite.WePoggin = result
	return result	
end

---------------------------------------------------------------

local costume = false

--Pog costume processing
function Poglite:OnPogMoment(itemCount)
	if Poglite:BlindCheck() then
	
		--print ("isRepStage=",isRepStage,"  roomItems=",roomItems)
		
		--Scan entities in room
		for i, entity in ipairs(Isaac.FindByType(5, 100)) do
			
			local id = entity.SubType
			local item = Isaac.GetItemConfig():GetCollectible(id)
			
			if item ~= nil then
				
				local visible = true
				--alt path blind
				visible = Poglite:AltBlindCheck(entity)
				
				--print(item.Name)
				--print(visible)
				
				if visible and item.Quality ~= nil then
				
					--Weird exceptions go first
					if id == 120 and CustomPogList["Odd Mushroom (Thin)"] == 1 then --thin odd mushroom
						--ADD COSTUME
						costume = Poglite:DoCostume(true)
						break
					elseif id == 121 and CustomPogList["Odd Mushroom (Large)"] == 1 then --large odd mushroom
						--ADD COSTUME
						costume = Poglite:DoCostume(true)
						break
					end
				
					--Check pog list
					if CustomPogList[item.Name] == 1 then
						--ADD COSTUME
						costume = Poglite:DoCostume(true)
						break
					end
					
					--Check item quality
					if item.Quality >= PogliteSettings["QualityThreshold"] and PogliteSettings["IgnoreQuality"] == false and CustomPogList[item.Name] ~= 2 then
						--ADD COSTUME
						costume = Poglite:DoCostume(true)
						break
					end				
				end
			end
		end
	end
end

---------------------------------------------------------------

--Scan items
function Poglite:ScanItems()
	local number = 0
	for i, entity in ipairs(Isaac.FindByType(5, 100)) do
		if entity.SubType > 0 then
			number = number + 1
		end
	end
	return number
end

--Check for curse of the blind
function Poglite:BlindCheck()
	if PogliteSettings["BlindBypass"] then
		return true
	end
	
	if game:GetLevel():GetCurses() & 64 ~= 0 then
		return false
	end
	return true
end

local questionMarkSprite = Sprite()
questionMarkSprite:Load("gfx/005.100_collectible.anm2",true)
questionMarkSprite:ReplaceSpritesheet(1,"gfx/items/collectibles/questionmark.png")
questionMarkSprite:LoadGraphics()

--Check for alt path blind items
function Poglite:AltBlindCheck(entity)
	if PogliteSettings["BlindBypass"] then
		return true
	end
	
	local entitySprite = entity:GetSprite()
	local name = entitySprite:GetAnimation()
	
	if name ~= "Idle" and name ~= "ShopIdle" then
		return true
	end
	
	questionMarkSprite:SetFrame(name,entitySprite:GetFrame())

	for i = -70,0,2 do
		local qcolor = questionMarkSprite:GetTexel(Vector(0,i),Vector(0,0),1,1)
		local ecolor = entitySprite:GetTexel(Vector(0,i),Vector(0,0),1,1)
		if qcolor.Red ~= ecolor.Red or qcolor.Green ~= ecolor.Green or qcolor.Blue ~= ecolor.Blue then
			return true
		end
	end

	for j = -3,3,2 do
		for i = -71,0,2 do
			local qcolor = questionMarkSprite:GetTexel(Vector(j,i),Vector(0,0),1,1)
			local ecolor = entitySprite:GetTexel(Vector(j,i),Vector(0,0),1,1)
			if qcolor.Red ~= ecolor.Red or qcolor.Green ~= ecolor.Green or qcolor.Blue ~= ecolor.Blue then
				return true
			end
		end
	end
	
	return false
end

---------------------------------------------------------------

local lastItemCount = 0
--Room update/pog reset
function Poglite:OnPogReset()
	lastItemCount = 0
	costume = Poglite:DoCostume(false)
end

--Game update
local firstLoad = true
function Poglite:OnGameUpdate()

	if firstLoad then
		Poglite:ModConfigInit()
		firstLoad = false
	end
	
	local currentItemCount = Poglite:ScanItems()
	
	if currentItemCount > lastItemCount then
		lastItemCount = currentItemCount
		Poglite:OnPogMoment(currentItemCount)
	elseif currentItemCount < lastItemCount then
		Poglite:OnPogReset()	
	end
end

--Item/Card updates
Poglite.ItemRollers = {
	CollectibleType.COLLECTIBLE_D6,
	CollectibleType.COLLECTIBLE_ETERNAL_D6,
	CollectibleType.COLLECTIBLE_D100,
	CollectibleType.COLLECTIBLE_D_INFINITY,
	CollectibleType.COLLECTIBLE_BLANK_CARD,
	CollectibleType.COLLECTIBLE_CLEAR_RUNE,
	CollectibleType.COLLECTIBLE_METRONOME,
	CollectibleType.COLLECTIBLE_SPINDOWN_DICE,
	CollectibleType.COLLECTIBLE_FLIP
}

function Poglite:OnUseItem(collectible)
	for k, v in pairs(Poglite.ItemRollers) do
		if collectible == v then
			--print(v)
			Poglite:OnPogReset()
			break
		end
	end	
end

Poglite.CardRollers = {
	Card.RUNE_PERTHRO,
	Card.CARD_DICE_SHARD,
	Card.RUNE_SHARD,
	Card.CARD_WILD,
	Card.CARD_SOUL_ISAAC,
	Card.CARD_SOUL_EDEN
}

function Poglite:OnUseCard(card)
	for k, v in pairs(Poglite.CardRollers) do
		if collectible == v then
			--print(v)
			Poglite:OnPogReset()
			break
		end
	end	
end

Poglite:AddCallback(ModCallbacks.MC_POST_UPDATE,Poglite.OnGameUpdate)
Poglite:AddCallback(ModCallbacks.MC_POST_NEW_ROOM,Poglite.OnPogReset)
Poglite:AddCallback(ModCallbacks.MC_USE_ITEM,Poglite.OnUseItem)
Poglite:AddCallback(ModCallbacks.MC_USE_CARD,Poglite.OnUseCard)